#include "common.h"
#include "PMDFileLoader.h"
#include "IReadFile.h"

CPMDFileLoader::CPMDFileLoader(void)
{
    init();
}

CPMDFileLoader::CPMDFileLoader(irr::video::IVideoDriver *dri)
{
    driver = dri;
    init();
}

CPMDFileLoader::~CPMDFileLoader(void)
{
    release();
}

void CPMDFileLoader::init()
{
    // 
    pmd = NULL;
    vmd = NULL;
    smesh = NULL;
    samesh = NULL;
    vecPMDMaterial.clear();
    int materialCount = 0;
}

void CPMDFileLoader::release()
{
    // I
    if (pmd != NULL)
    {
        delete pmd;
        pmd = NULL;
    }
    if (vmd != NULL)
    {
        delete vmd;
        vmd = NULL;
    }
    for (int i=vecPMDMaterial.size()-1; i>=0; i--)
    {
        delete vecPMDMaterial[i];
    }
    vecPMDMaterial.clear();
}

IAnimatedMesh* CPMDFileLoader::createMesh(io::IReadFile *file)
{
    // t@C\
    char filename[256], path[256];
    ::GetCurrentDirectoryA(MAX_PATH, path);
    sprintf_s(filename, sizeof(filename), "%s", file->getFileName());
    printf("filename=%s\n", filename);

    int i, ret;

    if (pmd != NULL)
    {
        release();
    }

    // PMDf[^̓ǂݍ
    pmd = new clsPMDFile;
    if ( !pmd->Open(file->getFileName()))
    {
        printf("Can't open PMD file! %s\n",
            file->getFileName());
        getchar();
        release();
        return NULL;
    }

    // TODO: VMDt@CǂݍŃAj[V쐬

    // PMDt@C̏ꍇAIuWFNg1A}eA
    // Ƃ̓}eAIuWFNgƂǂ
    // d͂邯ǊȒP
    materialCount = pmd->GetMaterialChunkSize();

    // eNX`ǂݍ݋yMaterial̍쐬
    // XLbVf[^̍쐬
    printf("SSkinMeshBuffer()\n");
    createMaterialData();

    // XLbVf[^ЂƂ̃bVɂ܂Ƃ߂
    smesh = new SMesh();
    for (int i=0; i<vecPMDMaterial.size(); i++)
    {
        smesh->addMeshBuffer((irr::scene::IMeshBuffer*)vecPMDMaterial[i]->ssmbuffer);
    }
    smesh->recalculateBoundingBox();
    printf("SMesh()\n");

    // Aj[VbV쐬Ԃ
    samesh = new SAnimatedMesh;
    samesh->addMesh((irr::scene::IMesh *)smesh);
    samesh->recalculateBoundingBox();
    printf("SAnimatedMesh()\n");

    return (irr::scene::IAnimatedMesh *)samesh;
}

bool CPMDFileLoader::isALoadableFileExtension(const c8 *fileName) const
{
    // gq`FbN
    std::string temp(fileName);
    if (temp.find(".pmd") == std::string::npos) return false;
    else return true;
}

void CPMDFileLoader::createMaterialData()
{
    int materialIndexCount = 0;
    char str[256], fileName[256];

    // }eA̐JԂ
    for (int i=0; i<materialCount; i++)
    {
        CPMDMaterial *sm = new CPMDMaterial();
        // PMDt@C̃}eAeNX`擾
        sprintf_s(str, 256, "%s", pmd->GetMaterialChunk()[i].textureFileName);
        // eNX`t@Cw肳Ăꍇ
        if (strcmp(str, "") != 0)
        {
            sprintf_s(fileName, 256, "..\\..\\media\\%s", str);
            // SMaterialɃeNX`t@Cݒ
            sm->setTexture(0, driver->getTexture(fileName));
            sm->BackfaceCulling = false;
        }
        // }eA̐ݒ
        sm->DiffuseColor.setRed(pmd->GetMaterialChunk()[i].diffuse.r*255.0);
        sm->DiffuseColor.setGreen(pmd->GetMaterialChunk()[i].diffuse.g*255.0);
        sm->DiffuseColor.setBlue(pmd->GetMaterialChunk()[i].diffuse.b*255.0);
        sm->DiffuseColor.setAlpha(pmd->GetMaterialChunk()[i].diffuse.a*255.0);
        sm->AmbientColor.setRed(pmd->GetMaterialChunk()[i].ambient.r*255.0);
        sm->AmbientColor.setGreen(pmd->GetMaterialChunk()[i].ambient.g*255.0);
        sm->AmbientColor.setBlue(pmd->GetMaterialChunk()[i].ambient.b*255.0);
        sm->SpecularColor.setRed(pmd->GetMaterialChunk()[i].specular.r*255.0);
        sm->SpecularColor.setGreen(pmd->GetMaterialChunk()[i].specular.g*255.0);
        sm->SpecularColor.setBlue(pmd->GetMaterialChunk()[i].specular.b*255.0);
        sm->Shininess = pmd->GetMaterialChunk()[i].shininess;

        // _̐ݒ
        sm->ssmbuffer   = new SPMDSkinMeshBuffer();
        // _Index
        sm->VertexCount = pmd->GetMaterialChunk()[i].nEdges;
        sm->IndexCount  = pmd->GetMaterialChunk()[i].nEdges;
        // _̓ǂݍ
        sm->Vertices    = new irr::f32[sm->VertexCount*3];
        sm->Indices     = new irr::u16[sm->IndexCount];
        sm->UData       = new irr::f32[sm->VertexCount];
        sm->VData       = new irr::f32[sm->VertexCount];
        sm->NVector     = new irr::f32[sm->VertexCount*3];
        sm->bone1       = new irr::u16[sm->VertexCount];
        sm->bone2       = new irr::u16[sm->VertexCount];
        sm->boneWeight  = new irr::u16[sm->VertexCount];
        
        // }eA̖ʐĂ
        for (int j=0; j<sm->IndexCount; j++)
        {
            int index = materialIndexCount + j;
            // Index
            sm->Indices[j] = pmd->GetIndexChunk()[index];
            // _W
            sm->Vertices[3*j+0] = pmd->GetVertexChunk()[sm->Indices[j]].x;
            sm->Vertices[3*j+1] = pmd->GetVertexChunk()[sm->Indices[j]].y;
            sm->Vertices[3*j+2] = pmd->GetVertexChunk()[sm->Indices[j]].z;
            // UV
            sm->UData[j] = pmd->GetVertexChunk()[sm->Indices[j]].tx;
            sm->VData[j] = pmd->GetVertexChunk()[sm->Indices[j]].ty;
            // @
            sm->NVector[3*j+0] = pmd->GetVertexChunk()[sm->Indices[j]].nx;
            sm->NVector[3*j+1] = pmd->GetVertexChunk()[sm->Indices[j]].ny;
            sm->NVector[3*j+2] = pmd->GetVertexChunk()[sm->Indices[j]].nz;
            // {[
            sm->bone1[j] = pmd->GetVertexChunk()[sm->Indices[j]].b1;
            sm->bone2[j] = pmd->GetVertexChunk()[sm->Indices[j]].b2;
            sm->boneWeight[j] = pmd->GetVertexChunk()[sm->Indices[j]].bw;
            // VIndexɕύX
            sm->Indices[j] = j;
        }
        materialIndexCount += sm->IndexCount;

        // SSkinMeshBuffer̍쐬
        sm->ssmbuffer->append(sm->Vertices, sm->VertexCount, sm->Indices, sm->IndexCount);
        sm->ssmbuffer->setNormals(sm->NVector);
        sm->ssmbuffer->MoveTo_2TCoords();
        sm->ssmbuffer->set2TCoords(sm->UData, sm->VData);
        sm->ssmbuffer->setMaterial(*sm);
        sm->ssmbuffer->recalculateBoundingBox();
        vecPMDMaterial.push_back(sm);
    }
}
